@@ -7,6 +7,7 @@ Tests for various parts of L{twisted.web}.
 
 import os
 import zlib
+from io import BytesIO
 
 from zope.interface import implementer
 from zope.interface.verify import verifyObject
@@ -26,6 +27,11 @@ from twisted.web.static import Data
 from twisted.logger import globalLogPublisher, LogLevel
 from twisted.test.proto_helpers import EventLoggingObserver
 
+from ._util import (
+    assertIsFilesystemTemporary,
+)
+
+
 
 class ResourceTests(unittest.TestCase):
     def testListEntities(self):
@@ -597,6 +603,39 @@ class RequestTests(unittest.TestCase):
         self.assertEqual(request.prePathURL(), b'http://example.com/foo%2Fbar')
 
 
+    def test_processingFailedNoTracebackByDefault(self):
+        """
+        By default, L{Request.processingFailed} does not write out the failure,
+        but give a generic error message, as L{Site.displayTracebacks} is
+        disabled by default.
+        """
+        logObserver = EventLoggingObserver.createWithCleanup(
+            self,
+            globalLogPublisher
+        )
+
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = server.Site(resource.Resource())
+        fail = failure.Failure(Exception("Oh no!"))
+        request.processingFailed(fail)
+
+        self.assertNotIn(b"Oh no!", request.transport.written.getvalue())
+        self.assertIn(
+            b"Processing Failed", request.transport.written.getvalue()
+        )
+        self.assertEquals(1, len(logObserver))
+
+        event = logObserver[0]
+        f = event["log_failure"]
+        self.assertIsInstance(f.value, Exception)
+        self.assertEquals(f.getErrorMessage(), "Oh no!")
+
+        # Since we didn't "handle" the exception, flush it to prevent a test
+        # failure
+        self.assertEqual(1, len(self.flushLoggedErrors()))
+
+
     def test_processingFailedNoTraceback(self):
         """
         L{Request.processingFailed} when the site has C{displayTracebacks} set
@@ -910,6 +949,59 @@ class RequestTests(unittest.TestCase):
         self.assertNotIn(b'content-type', response.lower())
 
 
+    def test_defaultSmallContentFile(self):
+        """
+        L{http.Request} creates a L{BytesIO} if the content length is small and
+        the site doesn't offer to create one.
+        """
+        request = server.Request(DummyChannel())
+        request.gotLength(100000 - 1)
+        self.assertIsInstance(request.content, BytesIO)
+
+
+    def test_defaultLargerContentFile(self):
+        """
+        L{http.Request} creates a temporary file on the filesystem if the
+        content length is larger and the site doesn't offer to create one.
+        """
+        request = server.Request(DummyChannel())
+        request.gotLength(100000)
+        assertIsFilesystemTemporary(self, request.content)
+
+
+    def test_defaultUnknownSizeContentFile(self):
+        """
+        L{http.Request} creates a temporary file on the filesystem if the
+        content length is not known and the site doesn't offer to create one.
+        """
+        request = server.Request(DummyChannel())
+        request.gotLength(None)
+        assertIsFilesystemTemporary(self, request.content)
+
+
+    def test_siteSuppliedContentFile(self):
+        """
+        L{http.Request} uses L{Site.getContentFile}, if it exists, to get a
+        file-like object for the request content.
+        """
+        lengths = []
+        contentFile = BytesIO()
+        site = server.Site(resource.Resource())
+
+        def getContentFile(length):
+            lengths.append(length)
+            return contentFile
+        site.getContentFile = getContentFile
+
+        channel = DummyChannel()
+        channel.site = site
+
+        request = server.Request(channel)
+        request.gotLength(12345)
+        self.assertEqual([12345], lengths)
+        self.assertIs(contentFile, request.content)
+
+
 
 class GzipEncoderTests(unittest.TestCase):
     def setUp(self):
